#pragma once
#include <wx/hashset.h>

const int DEFAULT_HASH_SIZE = 500;

WX_DECLARE_STRING_HASH_MAP(wxString, StringHashMap);
WX_DECLARE_STRING_HASH_MAP(StringHashMap *, StringHashMapMap);
WX_DECLARE_STRING_HASH_MAP(StringHashMap::iterator, StringHashMapIterator);

// TODO: This may need to be implemented as a template class, to support 
//       native data return types for nwnwx4 (map of ints, doubles, etc)
class HashData
{
public:
	HashData(int size);
	virtual ~HashData(void);
	
	// Empty the map
	void clear();

	// return number of elements in the map
	int getCount(); 

	// Add 'value' at 'key'.  Note that the results of this on 
	// any in-progress iterations are unpredictable.
	void add(const wxString & key, wxString & value);

	// Remove 'key' and its associated value.  Note that the results of this on 
	// any in-progress iterations are unpredictable.
	void remove(const wxString & key);
	
	// if 'key' exists, populates 'value' with associatd value. 
	// otherwise, it returns false. 
	bool getValue(const wxString & key, wxString & value);
	// initializes the key iterator to the first position in the map
	// this is done even if another iteration is already in progress. 
	// returns false if the map is empty; otherwise "key" will contain the key.
	bool getFirstKey(wxString & key); 

	// if possible, returns the next key in the map.  Returns false if there
	// is no next key (or getFirstKey not called first), otherwise populates 'key'
	bool getNextKey(wxString & key); 

	// returns "true" if there is a next key in the map. 
	bool hasNextKey(); 

	// returns "true" if there is a next value in the map. 
	bool hasNextValue();

	// if it exists, populates 'key' with the value iterator's 
	// current key.  If none exists, returns false; 
	bool getCurrentKey(wxString & key);

	// initializes the value iterator to the first position in the map
	// this is done even if another iteration is already in progress. 
	// returns false if the map is empty. Otherwise "value" will contain the value.
	bool getFirstValue(wxString & value); 

	// if possible, returns the next value in the map.  Returns false if there
	// is no next value(or getFirstValue not called first), otherwise populates 'value'
	bool getNextValue(wxString & value);

	// if it exists, populates 'value' with the value iterator's 
	// current value. If none exists, returns false; 
	bool getCurrentValue(wxString & value);

	

private: 
	StringHashMap map;
	// Holds the current position in this map's key iterations
	StringHashMap::iterator keyIterator;

	// Holds the current position in this map's value iterations
	StringHashMap::iterator valueIterator;



};

WX_DECLARE_STRING_HASH_MAP(HashData *, MasterHashMap);

