/***************************************************************************
    NWNX HashSetPlugin - HashSetPluginity functions plugin
    Copyright (C) 2006 Marc Paradise [Grinning Fool, gf@grinningfool.org] 
	Based upon interface defined in NWNX2's HashSetPlugin implementation by 


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 ***************************************************************************/
#if !defined(HashSetPlugin_H_INCLUDED)
#define HashSetPlugin_H_INCLUDED

#define DLLEXPORT extern "C" __declspec(dllexport)

#include "../plugin.h"
#include <wx/hashset.h>
#include <wx/tokenzr.h>
#include "HashData.h"


class wxLogNWNX;
class HashItem;


class HashSetPlugin : public Plugin
{
public:
	HashSetPlugin();
	~HashSetPlugin();

	bool Init(TCHAR* nwnxhome);  
	const char* DoRequest(char *gameObject, char* request, char* parameters);
	void GetFunctionClass(TCHAR* fClass);

private:
	enum Command { 
		create, 
		destroy, 
		isValid,
		isExisting,
		insert,
		lookup,
		del,
		getFirstKey,
		getNextKey,
		hasNextKey,
		getFirstValue,
		getNextValue,
		hasNextValue,
		getSize,
		loadFromFile,
		startMappedToken,
		nextMappedToken
	};
	// THought: Plugins can register their known commands with the base plugin class. 
	//       plugin class invokes by passing in the int value of the command. 
	//       removes the need for each plugin to implement its own text parsing. 
	// registerCommand("TEXT", const int value) 
	// And the callback: 
	// executeCommand(const int, params, data)

	wxLogNWNX* logger;
	WX_DECLARE_STRING_HASH_MAP(Command, CommandMap);
	WX_DECLARE_STRING_HASH_MAP(HashData *, HashMaps);
	HashMaps m_maps;

	CommandMap m_knownCommands;
	char * m_pData;
	wxStringTokenizer * m_pMapTokenizer;
	wxString m_returnData; 
	
	HashData * getMapFromFile(const wxString & filename, const wxString & sep);
	inline void addMapEntry(HashData * map, wxString & key, wxString & value);
	inline void deleteMap(const wxString& name);
	inline HashData * findMap(const wxString & name);
	void newTokenSearch(const char * pszData);
	void nextTokenizerMatch(HashData * map, char * output);

};

#endif